package me.totalfreedom.totalfreedommod.rank;

import net.md_5.bungee.api.ChatColor;
import org.jetbrains.annotations.NotNull;

public enum Rank implements Displayable
{
    NON_OP("a", "Non-Op", "tfm.default", "", "Non-Ops", ChatColor.WHITE, null, false, false),
    OP("an", "Operator", "tfm.op", "OP", "Operators", ChatColor.GREEN, null, false, false),
    ADMIN("an", "Admin", "tfm.admin", "Admin", "Administrators", ChatColor.DARK_GREEN, org.bukkit.ChatColor.DARK_GREEN, true, true),
    SENIOR_ADMIN("a", "Senior Admin", "tfm.senior", "SrA", "Senior Administrators", ChatColor.GOLD, org.bukkit.ChatColor.GOLD, true, true),
    ADMIN_CONSOLE("the", "Console", "tfm.rcon", "Console", "Administrator Consoles", ChatColor.DARK_PURPLE, null, false, false),
    SENIOR_CONSOLE("the", "Console", "tfm.rcon.senior", "Console", "Senior Consoles", ChatColor.DARK_PURPLE, null, false, false);

    private final String permission;

    private final String name;

    private final String abbr;
    private final String plural;
    private final String article;

    private final String tag;

    private final String coloredTag;

    private final ChatColor color;

    private final org.bukkit.ChatColor teamColor;

    private final boolean hasTeam;

    private final boolean hasDefaultLoginMessage;

    Rank(String article, String name, String permission, String abbr, String plural, ChatColor color, org.bukkit.ChatColor teamColor, Boolean hasTeam, Boolean hasDefaultLoginMessage)
    {
        this.permission = permission;
        this.name = name;
        this.abbr = abbr;
        this.plural = plural;
        this.article = article;
        this.tag = abbr.isEmpty() ? "" : "[" + abbr + "]";
        this.coloredTag = abbr.isEmpty() ? "" : ChatColor.DARK_GRAY + "[" + color + abbr + ChatColor.DARK_GRAY + "]" + color;
        this.color = color;
        this.teamColor = teamColor;
        this.hasTeam = hasTeam;
        this.hasDefaultLoginMessage = hasDefaultLoginMessage;
    }

    public static Rank findRank(String string)
    {
        try
        {
            return Rank.valueOf(string.toUpperCase());
        } catch (Exception ignored)
        {
        }

        return Rank.NON_OP;
    }

    @Override
    public String getColoredName()
    {
        return color + name;
    }

    @Override
    public String getColoredLoginMessage()
    {
        return article + " " + color + name;
    }

    @Override
    public boolean hasTeam()
    {
        return hasTeam;
    }

    @Override
    public boolean hasDefaultLoginMessage()
    {
        return hasDefaultLoginMessage;
    }

    @Override
    public String getAbbr()
    {
        return abbr;
    }

    public String getPlural()
    {
        return plural;
    }

    public boolean isConsole()
    {
        return getPermission().equalsIgnoreCase("tfm.rcon") ||
                getPermission().equalsIgnoreCase("tfm.rcon.senior"); // check for either console.
    }

    public int getLevel()
    {
        return ordinal();
    }

    public boolean isAtLeast(@NotNull Rank rank)
    {
        if (getLevel() < rank.getLevel())
        {
            return false;
        }

        if (!hasConsoleVariant() || !rank.hasConsoleVariant())
        {
            return true;
        }

        assert getConsoleVariant() != null;
        assert rank.getConsoleVariant() != null;
        return getConsoleVariant().getLevel() >= rank.getConsoleVariant().getLevel();
    }

    public boolean isAdmin()
    {
        return getPermission().equalsIgnoreCase("tfm.admin") ||
                getPermission().equalsIgnoreCase("tfm.rcon");
    }

    public boolean hasConsoleVariant()
    {
        return getConsoleVariant() != null;
    }

    public Rank getConsoleVariant()
    {
        return switch (this)
                {
                    case ADMIN, ADMIN_CONSOLE -> ADMIN_CONSOLE;
                    case SENIOR_ADMIN, SENIOR_CONSOLE -> SENIOR_CONSOLE;
                    default -> null;
                };
    }

    public String getPermission()
    {
        return permission;
    }

    @Override
    public String getName()
    {
        return name;
    }

    @Override
    public String getArticle()
    {
        return article;
    }

    @Override
    public String getTag()
    {
        return tag;
    }

    @Override
    public String getColoredTag()
    {
        return coloredTag;
    }

    @Override
    public ChatColor getColor()
    {
        return color;
    }

    @Override
    public org.bukkit.ChatColor getTeamColor()
    {
        return teamColor;
    }

    public boolean isHasTeam()
    {
        return hasTeam;
    }

    public boolean isHasDefaultLoginMessage()
    {
        return hasDefaultLoginMessage;
    }
}